@@ -1,11 +1,10 @@
 package Var::Pairs;
 
-our $VERSION = '0.002004';
+our $VERSION = '0.003004';
 
 use 5.014;
 use warnings;
 no if $] >= 5.018, warnings => "experimental::smartmatch";
-no if $] >= 5.020, warnings => "experimental::autoderef";
 use Carp;
 use Devel::Callsite;
 
@@ -117,7 +116,7 @@ sub pairs (+) {
     }
 
     # Uniquely identify this call, according to its lexical context...
-    my $ID = callsite() . $container_ref;
+    my $ID = callsite() . context() . $container_ref;
 
     # Short-circuit if this is a repeated call...
     if (!wantarray && $iterator_for{$ID}) {
@@ -127,7 +126,7 @@ sub pairs (+) {
     # Generate the list of pairs, according to the container type...
     my $container_is_array = $container_type eq 'ARRAY';
     my @pairs = map { Var::Pairs::Pair->new($_, $container_ref, $container_is_array ? 'array' : 'hash') }
-                    $container_is_array ? 0..$#{$container_ref} : keys $container_ref;
+                    $container_is_array ? 0..$#{$container_ref} : keys %{$container_ref};
 
     # Return them all in list context...
     return @pairs;
@@ -141,7 +140,7 @@ sub each_pair (+) {
     my ($container_ref) = @_;
 
     # Uniquely identify this call, according to its lexical context...
-    my $ID = callsite() . $container_ref;
+    my $ID = callsite() . context() . $container_ref;
 
     # Build an iterator...
     $iterator_for{$ID} //= [ &pairs ];
@@ -168,7 +167,7 @@ sub kvs (+) {
     }
 
     # Uniquely identify this call, according to its lexical context...
-    my $ID = callsite() . $container_ref;
+    my $ID = callsite() . context() . $container_ref;
 
     # Return the key/value list, according to the container type...
     if ($container_type eq 'ARRAY') {
@@ -183,7 +182,7 @@ sub each_kv (+) {
     my ($container_ref) = @_;
 
     # Uniquely identify this call, according to its lexical context and iteration target...
-    my $ID = callsite() . $container_ref;
+    my $ID = callsite() . context() . $container_ref;
 
     # Build an iterator...
     $iterator_for{$ID} //= [ &kvs ];
@@ -223,11 +222,11 @@ sub _invert {
 
     given (ref($var_ref) || 'SCALAR') {
         when ('HASH') {
-            for my $key (keys $var_ref) {
+            for my $key (keys %{$var_ref}) {
                 my $values = $var_ref->{$key};
                 for my $value ( ref $values eq 'ARRAY' ? @$values : $values ) {
                     $inversion{$value} //= [];
-                    push $inversion{$value}, $key;
+                    push @{$inversion{$value}}, $key;
                 }
             }
         }
@@ -236,7 +235,7 @@ sub _invert {
                 my $values = $var_ref->[$key];
                 for my $value ( ref $values eq 'ARRAY' ? @$values : $values ) {
                     $inversion{$value} //= [];
-                    push $inversion{$value}, $key;
+                    push @{$inversion{$value}}, $key;
                 }
             }
         }
@@ -253,7 +252,7 @@ sub _get_each_pair {
     my $ID = shift;
 
     # Iterator the requested iterator...
-    my $each_pair = shift $iterator_for{$ID};
+    my $each_pair = shift @{$iterator_for{$ID}};
 
     # If nothing was left to iterate, clean up the empty iterator...
     if (!defined $each_pair) {
@@ -267,7 +266,7 @@ sub _get_each_kv {
     my $ID = shift;
 
     # Iterator the requested iterator...
-    my @each_kv = splice $iterator_for{$ID}, 0, 2;
+    my @each_kv = splice @{$iterator_for{$ID}}, 0, 2;
 
     # If nothing was left to iterate, clean up the empty iterator...
     if (!@each_kv) {
@@ -278,81 +277,8 @@ sub _get_each_kv {
     return wantarray ? @each_kv : $each_kv[0];
 }
 
-# Class implementing each key/value pair...
-package Var::Pairs::Pair {
-    use Scalar::Util qw< looks_like_number >;
-    use Data::Alias;
-    use Carp;
-
-    # Each pair object has two attributes...
-    my @key_for;
-    my @value_for;
-    my @freed;
-
-    # Accessors for the attributes (value is read/write)...
-    sub value :lvalue { $value_for[${shift()}] }
-    sub index         {   $key_for[${shift()}] }
-    sub key           {   $key_for[${shift()}] }
-    sub kv            { my $self = shift;  $key_for[$$self], $value_for[$$self] }
-
-    # The usual inside-out constructor...
-    sub new {
-        my ($class, $key, $container_ref, $container_type) = @_;
-
-        # Create a scalar based object...
-        my $scalar = @key_for;
-        my $new_obj = bless \$scalar, $class;
-
-        # Initialize its attributes (value needs to be an alias to the original)...
-        $key_for[$scalar] = $key;
-        alias $value_for[$scalar] = $container_type eq 'array' ? $container_ref->[$key]
-                                  : $container_type eq 'none'  ? $_[2]
-                                  :                              $container_ref->{$key};
-        $freed[$scalar] = 0;
-
-        return $new_obj;
-    }
-
-    # Type coercions...
-    use overload (
-        # As a string, a pair is just: key => value
-        q{""}   => sub {
-            my $self = shift;
-            my $value = $value_for[$$self];
-            $value = ref $value                ? ref $value
-                   : looks_like_number($value) ? $value
-                   :                             qq{"$value"};
-            return "$key_for[$$self] => $value";
-        },
-
-        # Can't numerify a pair (make it a hanging offence)...
-        q{0+}   => sub { croak "Can't convert Pair(".shift.") to a number" },
-
-        # All pairs are true (just as in Perl 6)...
-        q{bool} => sub { !!1 },
-
-        # Everything else as normal...
-        fallback => 1,
-    );
-
-    sub DESTROY {
-        my $self = shift;
-
-        # Mark current storage as reclaimable...
-        $freed[$$self] = 1;
-
-        # Reclaim everything possible...
-        if ($freed[$#freed]) {
-            my $free_from = $#freed;
-            while ($free_from >= 0 && $freed[$free_from]) {
-                $free_from--;
-            }
-            splice @key_for,   $free_from+1;
-            splice @value_for, $free_from+1;
-            splice @freed,     $free_from+1;
-        }
-    }
-}
+use if $] <  5.022, 'Var::Pairs::Pair_DataAlias';
+use if $] >= 5.022, 'Var::Pairs::Pair_BuiltIn';
 
 1; # Magic true value required at end of module
 __END__
@@ -365,7 +291,7 @@ Var::Pairs - OO iterators and pair constructors for variables
 
 =head1 VERSION
 
-This document describes Var::Pairs version 0.002004
+This document describes Var::Pairs version 0.003004
 
 
 =head1 SYNOPSIS
@@ -848,7 +774,7 @@ The module requires Perl 5.014 and the following modules:
 
 =item Devel::Callsite
 
-=item Data::Alias
+=item Data::Alias (under Perl 5.20 and earlier)
 
 =item PadWalker
 
